@@ -1,14 +1,16 @@
 package SVK::Command::Commit;
 use strict;
-our $VERSION = $SVK::VERSION;
+use SVK::Version;  our $VERSION = $SVK::VERSION;
 use base qw( SVK::Command );
 use constant opt_recursive => 1;
 use SVK::XD;
 use SVK::I18N;
 use SVK::Editor::Status;
 use SVK::Editor::Sign;
+use SVK::Command::Sync;
 use SVK::Util qw( HAS_SVN_MIRROR get_buffer_from_editor slurp_fh read_file
-		  find_svm_source tmpfile abs2rel find_prev_copy );
+		  find_svm_source tmpfile abs2rel find_prev_copy from_native to_native
+		  get_encoder );
 
 sub options {
     ('m|message=s'  => 'message',
@@ -16,8 +18,9 @@ sub options {
      'C|check-only' => 'check_only',
      'S|sign'	  => 'sign',
      'P|patch=s'  => 'patch',
-     'import',	  => 'import',
-     'direct',	  => 'direct',
+     'import'	  => 'import',
+     'encoding=s' => 'encoding',
+     'direct'	  => 'direct',
     );
 }
 
@@ -44,15 +47,6 @@ sub under_mirror {
     HAS_SVN_MIRROR and SVN::Mirror::is_mirrored ($target->{repos}, $target->{path});
 }
 
-sub check_mirrored_path {
-    my ($self, $target) = @_;
-    if ($self->under_mirror ($target)) {
-	print loc ("%1 is under mirrored path.\n", $target->{depotpath});
-	return;
-    }
-    return 1;
-}
-
 sub fill_commit_message {
     my $self = shift;
     if ($self->{message_file}) {
@@ -65,9 +59,17 @@ sub fill_commit_message {
 sub get_commit_message {
     my ($self, $msg) = @_;
     $self->fill_commit_message;
-    return if defined $self->{message};
     $self->{message} = get_buffer_from_editor
-	(loc('log message'), $self->message_prompt, join ("\n", $msg || '', $self->message_prompt, ''), 'commit');
+	(loc('log message'), $self->message_prompt,
+	 join ("\n", $msg || '', $self->message_prompt, ''), 'commit')
+	    unless defined $self->{message};
+    $self->decode_commit_message;
+}
+
+sub decode_commit_message {
+    my $self = shift;
+    eval { from_native ($self->{message}, 'commit message', $self->{encoding}); 1 }
+	or die $@.loc("try --encoding.\n");
 }
 
 # XXX: This should just return Editor::Dynamic objects
@@ -128,6 +130,7 @@ sub get_editor {
 	}
 	else {
 	    print loc("Merging back to mirror source %1.\n", $m->{source});
+	    $m->{lock_message} = SVK::Command::Sync::lock_message ();
 	    $m->{config} = $self->{svnconfig};
 	    $m->{revprop} = ['svk:signature'];
 	    ($base_rev, $editor) = $m->get_merge_back_editor
@@ -148,7 +151,7 @@ sub get_editor {
     my $yrev = $fs->youngest_rev;
     my $root = $fs->revision_root ($yrev);
 
-    %cb = SVK::Editor::Merge::cb_for_root
+    %cb = SVK::Editor::Merge->cb_for_root
 	($root, $target->{path}, defined $base_rev ? $base_rev : $yrev);
 
     if ($self->{patch}) {
@@ -191,7 +194,9 @@ sub get_editor {
     }
 
     unless ($self->{check_only}) {
-	for ($SVN::Error::FS_TXN_OUT_OF_DATE, $SVN::Error::FS_ALREADY_EXISTS) {
+	for ($SVN::Error::FS_TXN_OUT_OF_DATE,
+	     $SVN::Error::FS_ALREADY_EXISTS,
+	     $SVN::Error::FS_NOT_DIRECTORY) {
 	    # XXX: there's no copath info here
 	    $self->msg_handler ($_, $m ? "Please sync mirrored path $target->{path} first."
 				       : "Please update checkout first.");
@@ -224,10 +229,12 @@ sub get_committable {
     print $fh "\n", $self->target_prompt, "\n" if $fh;
 
     my $targets = [];
+    my $encoder = get_encoder;
     my $statuseditor = SVK::Editor::Status->new
 	( notify => SVK::Notify->new
 	  ( cb_flush => sub {
 		my ($path, $status) = @_;
+		to_native ($path, 'path', $encoder);
 		my $copath = $target->copath ($path);
 		push @$targets, [$status->[0] || ($status->[1] ? 'P' : ''),
 				 $copath];
@@ -269,7 +276,7 @@ sub get_committable {
 	die loc("No targets to commit.\n") if $#{$targets} < 0;
 	unlink $file;
     }
-
+    $self->decode_commit_message;
     return [sort {$a->[1] cmp $b->[1]} @$targets];
 }
 
@@ -310,15 +317,17 @@ sub committed_commit {
 	}
 	my $root = $fs->revision_root ($rev);
 	# update keyword-translated files
+	my $encoder = get_encoder;
 	for (@$targets) {
 	    my ($action, $copath) = @$_;
 	    next if $action eq 'D' || -d $copath;
 	    my $path = $target->{path};
 	    $path = '' if $path eq '/';
 	    my $dpath = abs2rel($copath, $target->{copath} => $path, '/');
+	    from_native ($dpath, 'path', $encoder);
 	    my $prop = $root->node_proplist ($dpath);
 	    my $layer = SVK::XD::get_keyword_layer ($root, $dpath, $prop);
-	    my $eol = SVK::XD::get_eol_layer ($root, $dpath, $prop, '>');
+	    my $eol = SVK::XD::get_eol_layer ($prop, '>');
 	    # XXX: can't bypass eol translation when normalization needed
 	    next unless $layer || ($eol ne ':raw' && $eol ne ' ');
 
@@ -401,7 +410,7 @@ sub run_delta {
 		my ($source_path, $source_rev) = $self->{xd}->_copy_source ($entry, $cotarget);
 		($source_path, $source_rev) = ($revtarget, $entry->{revision})
 		    unless defined $source_path;
-		return $revcache{$source_rev} if exists $revcache{corev};
+		return $revcache{$source_rev} if exists $revcache{$source_rev};
 		my $rev = ($fs->revision_root ($source_rev)->node_history ($source_path)->prev (0)->location)[1];
 		$revcache{$source_rev} = $cb{mirror}->find_remote_rev ($rev);
 	    }) : ());
@@ -422,10 +431,12 @@ SVK::Command::Commit - Commit changes to depot
 
 =head1 OPTIONS
 
- -m [--message] arg     : specify commit message ARG
+ -m [--message] MESSAGE	: specify commit message MESSAGE
  -C [--check-only]      : try operation but make no changes
- -P [--patch] arg       : instead of commit, save this change as a patch
+ -P [--patch] NAME	: instead of commit, save this change as a patch
  -S [--sign]            : sign this change
+ -N [--non-recursive]   : operate on single directory only
+ --encoding ENC         : treat value as being in charset encoding ENC
  --import               : import mode; automatically add and delete nodes
  --direct               : commit directly even if the path is mirrored
 
