@@ -2,7 +2,7 @@ package SVK::Command::Copy;
 use strict;
 use SVK::Version;  our $VERSION = $SVK::VERSION;
 use base qw( SVK::Command::Mkdir );
-use SVK::Util qw( get_anchor get_prompt abs2rel splitdir );
+use SVK::Util qw( get_anchor get_prompt abs2rel splitdir is_uri );
 use SVK::I18N;
 
 sub options {
@@ -17,12 +17,23 @@ sub parse_arg {
     push @arg, '' if @arg == 1;
 
     my $dst = pop(@arg);
-    my @src = (map {$self->arg_co_maybe ($_)} @arg);
+    die loc ("Copy destination can't be URI.\n")
+	if is_uri ($dst);
+
+    die loc ("More than one URI found.\n")
+	if (grep {is_uri($_)} @arg) > 1;
+    my @src;
 
     if ( my $target = eval { $self->arg_co_maybe ($dst) }) {
         $dst = $target;
+	# don't allow new uri in source when target is copath
+	@src = (map {$self->arg_co_maybe
+			 ($_, $dst->{copath}
+			  ? loc ("path '%1' is already a checkout", $dst->{report})
+			  : undef)} @arg);
     }
     else {
+	@src = (map {$self->arg_co_maybe ($_)} @arg);
         # Asking the user for copy destination.
         # In this case, first magically promote ourselves to "cp -p".
         # (otherwise it hurts when user types //deep/directory/name)
@@ -36,7 +47,7 @@ sub parse_arg {
 
         my $path = $self->prompt_depotpath("copy", $default);
 
-        if ($dst =~ /^\.?$/) {
+        if ($dst eq '.') {
             $self->{_checkout_path} = (splitdir($path))[-1];
         }
         else {
@@ -60,9 +71,10 @@ sub handle_co_item {
     my $xdroot = $dst->root ($self->{xd});
     die loc ("Path %1 does not exist.\n", $src->{path})
 	if $src->root->check_path ($src->{path}) == $SVN::Node::none;
-    die loc ("Path %1 already exists.\n", $dst->{copath})
-	if -e $dst->{copath};
     my ($copath, $report) = @{$dst}{qw/copath report/};
+    die loc ("Path %1 already exists.\n", $copath)
+	if -e $copath;
+    my $entry = $self->{xd}{checkout}->get ($copath);
     $src->normalize;
     $src->anchorify; $dst->anchorify;
     # if SVK::Merge could take src being copath to do checkout_delta
@@ -74,7 +86,8 @@ sub handle_co_item {
 
     $self->{xd}{checkout}->store_recursively ($copath, {'.schedule' => undef,
 							'.newprop' => undef});
-    $self->{xd}{checkout}->store ($copath, {'.schedule' => 'add',
+    # XXX: can the scheudle be something other than delete ?
+    $self->{xd}{checkout}->store ($copath, {'.schedule' => $entry->{'.schedule'} ? 'replace' : 'add',
 					    scheduleanchor => $copath,
 					    '.copyfrom' => $src->path,
 					    '.copyfrom_rev' => $src->{revision}});
@@ -83,6 +96,10 @@ sub handle_co_item {
 sub handle_direct_item {
     my ($self, $editor, $anchor, $m, $src, $dst) = @_;
     $src->normalize;
+    # if we have targets, ->{path} must exist
+    if (!$self->{parent} && $dst->{targets} && !$dst->root->check_path ($dst->{path})) {
+	die loc ("Parent directory %1 doesn't exist, use -p.\n", $dst->{report});
+    }
     my ($path, $rev) = @{$src}{qw/path revision/};
     if ($m) {
 	$path =~ s/^\Q$m->{target_path}\E/$m->{source}/;
@@ -135,9 +152,12 @@ sub run {
     # XXX: check dst to see if the copy is obstructured or missing parent
     my $fs = $dst->{repos}->fs;
     if ($dst->{copath}) {
-	# XXX: check if dst is versioned
-	return loc("%1 is not a directory.\n", $dst->{copath})
+	return loc("%1 is not a directory.\n", $dst->{report})
 	    if $#src > 0 && !-d $dst->{copath};
+	return loc("%1 is not a versioned directory.\n", $dst->{report})
+	    if -d $dst->{copath} &&
+		!($dst->root($self->{xd})->check_path ($dst->path) ||
+		  $self->{xd}{checkout}->get ($dst->{copath})->{'.schedule'});
 	my @cpdst;
 	for (@src) {
 	    my $cpdst = $dst->new;
@@ -192,10 +212,10 @@ SVK::Command::Copy - Make a versioned copy
 
 =head1 OPTIONS
 
- -r [--revision] arg    : act on revision ARG instead of the head revision
- -m [--message] arg     : specify commit message ARG
+ -r [--revision] REV	: act on revision REV instead of the head revision
+ -m [--message] MESSAGE : specify commit message MESSAGE
  -p [--parent]          : create intermediate directories as required
- -P [--patch] arg       : instead of commit, save this change as a patch
+ -P [--patch] NAME	: instead of commit, save this change as a patch
  -C [--check-only]      : try operation but make no changes
  -S [--sign]            : sign this change
 
