@@ -1,6 +1,6 @@
 package SVK::Command::Push;
 use strict;
-our $VERSION = $SVK::VERSION;
+use SVK::Version;  our $VERSION = $SVK::VERSION;
 
 use base qw( SVK::Command::Smerge );
 use SVK::XD;
@@ -9,6 +9,7 @@ sub options {
     ('f|from=s'         => 'from_path',
      'l|lump'           => 'lump',
      'C|check-only'     => 'check_only',
+     'summary'		=> 'summary',
      'S|sign'	        => 'sign',
      'P|patch=s'        => 'patch',
      'verbatim'		=> 'verbatim',
@@ -53,10 +54,10 @@ SVK::Command::Push - Move changes into another repository
 
 =head1 OPTIONS
 
- -f [--from] arg        : push from the specified path
+ -f [--from] PATH	: push from the specified path
  -l [--lump]            : merge everything into a single commit log
  -C [--check-only]      : try operation but make no changes
- -P [--patch] arg       : instead of commit, save this change as a patch
+ -P [--patch] NAME	: instead of commit, save this change as a patch
  -S [--sign]            : sign this change
  --verbatim             : verbatim merge log without indents and header
 
