@@ -26,6 +26,25 @@ sub copy_notify {
     return find_local_mirror ($m->{repos}, $m->{rsource_uuid}, $from_path, $from_rev);
 }
 
+sub lock_message {
+    my $i = 0;
+    sub {
+	my ($mirror, $what, $who) = @_;
+	print loc("Waiting for %1 lock on %2: %3.\n", $what, $mirror->{target_path}, $who);
+	if (++$i % 3) {
+	    print loc ("
+The mirror is currently locked. This might be because the mirror is
+in the middle of a sensitive operation or because a process holding
+the lock hung or died.  To check if the mirror lock is stalled,  see 
+if $who is a running, valid process
+
+If the mirror lock is stalled, please interrupt this process and run:
+    svk mirror --unlock %1
+", $mirror->{target_path});
+	}
+    }
+}
+
 sub run {
     my ($self, @arg) = @_;
     die loc("cannot load SVN::Mirror") unless HAS_SVN_MIRROR;
@@ -61,6 +80,7 @@ sub run {
 				  pool => SVN::Pool->new,
 				  config => $self->{svnconfig},
 				  cb_copy_notify => sub { $self->copy_notify (@_) },
+				  lock_message => lock_message(),
 				  revprop => ['svk:signature'],
 				  get_source => 1, skip_to => $self->{skip_to});
 	$m->init ();
@@ -99,8 +119,8 @@ SVK::Command::Sync - Synchronize a mirrored depotpath
 =head1 OPTIONS
 
  -a [--all]             : synchronize all mirrored paths
- -s [--skipto] arg      : start synchronization at revision ARG
- -t [--torev] arg       : stop synchronization at revision ARG
+ -s [--skipto] REV	: start synchronization at revision REV
+ -t [--torev] REV	: stop synchronization at revision REV
 
 =head1 AUTHORS
 
