@@ -1,11 +1,12 @@
 package SVK::Editor::Merge;
 use strict;
-our $VERSION = $SVK::VERSION;
+use SVK::Version;  our $VERSION = $SVK::VERSION;
+
+require SVN::Delta;
 our @ISA = qw(SVN::Delta::Editor);
-use SVK::Notify;
 use SVK::I18N;
-use SVK::Util qw( slurp_fh md5_fh get_anchor tmpfile devnull );
-use IO::Digest;
+use autouse 'SVK::Util' => qw( slurp_fh md5_fh get_anchor tmpfile devnull );
+
 use constant FH => 0;
 use constant FILENAME => 1;
 use constant CHECKSUM => 2;
@@ -130,7 +131,7 @@ use Digest::MD5 qw(md5_hex);
 use File::Compare ();
 
 sub cb_for_root {
-    my ($root, $anchor, $base_rev) = @_;
+    my ($class, $root, $anchor, $base_rev) = @_;
     return ( cb_exist =>
 	     sub { my $path = $anchor.'/'.shift;
 		   return $root->check_path ($path);
@@ -172,11 +173,11 @@ sub cb_for_root {
 # translate the path before passing to cb_*
 sub cb_translate {
     my ($cb, $translate) = @_;
-    for (qw/cb_exist cb_rev cb_conflict cb_localmod cb_dirdelta/) {
+    for (qw/cb_exist cb_rev cb_conflict cb_localmod cb_localprop cb_dirdelta/) {
 	my $sub = $cb->{$_};
 	next unless $sub;
 	$cb->{$_} = sub { my $path = shift; $translate->($path);
-			  $sub->($path, @_)};
+			  unshift @_, $path; goto &$sub };
     }
 }
 
@@ -265,8 +266,11 @@ sub ensure_close {
     $self->{notify}->flush ($path, 1);
     $self->{cb_closed}->($path, $checksum, $pool)
         if $self->{cb_closed};
-    $self->{cb_merged}->($self->{storage}, $self->{storage_baton}{$path}, 'file', $pool)
-	if $path eq $self->{target} && $self->{changes} && $self->{cb_merged};
+
+    if ($path eq $self->{target} && $self->{changes} && $self->{cb_merged}) {
+	$self->ensure_open ($path);
+	$self->{cb_merged}->($self->{storage}, $self->{storage_baton}{$path}, 'file', $pool);
+    }
 
     if (my $baton = $self->{storage_baton}{$path}) {
 	$self->{storage}->close_file ($baton, $checksum, $pool);
@@ -292,13 +296,19 @@ sub cleanup_fh {
 }
 
 sub prepare_fh {
-    my ($self, $fh) = @_;
-    # XXX: need to respect eol-style here?
+    my ($self, $fh, $eol) = @_;
     for my $name (qw/base new local/) {
 	my $entry = $fh->{$name};
 	next unless $entry->[FH];
-	next if $entry->[FILENAME];
+	# if there's eol translation required, we can't use the
+	# prepared tmp files.
+	if ($entry->[FILENAME]) {
+	    next unless $eol;
+	    # reopen the tmp file, since apply_textdelta closes it
+	    open $entry->[FH], $entry->[FILENAME];
+	}
 	my $tmp = [tmpfile("$name-"), $entry->[CHECKSUM]];
+	binmode $tmp->[FH], $eol if $eol;
 	slurp_fh ($entry->[FH], $tmp->[FH]);
 	close $entry->[FH];
 	$entry = $fh->{$name} = $tmp;
@@ -306,34 +316,43 @@ sub prepare_fh {
     }
 }
 
+sub _retrieve_base
+{
+    my ($self, $path, $pool) = @_;
+    my @base = tmpfile('base-');
+    $path = "$self->{base_anchor}/$path" if $self->{base_anchor};
+    slurp_fh ($self->{base_root}->file_contents ($path, $pool), $base[FH]);
+    seek $base[FH], 0, 0;
+    return @base;
+}
+
 sub apply_textdelta {
     my ($self, $path, $checksum, $pool) = @_;
     return unless $path;
 
     my $info = $self->{info}{$path};
     my $fh = $info->{fh} = {};
-    my ($base);
     if (($pool = $info->{fpool}) &&
 	($fh->{local} = $self->{cb_localmod}->($path, $checksum || '', $pool))) {
 	# retrieve base
 	unless ($info->{addmerge}) {
-	    $fh->{base} = [tmpfile('base-')];
-	    $path = "$self->{base_anchor}/$path" if $self->{base_anchor};
-	    slurp_fh ($self->{base_root}->file_contents ($path, $pool),
-		      $fh->{base}[FH]);
-	    $base = $fh->{base}[FH];
-	    seek $base, 0, 0;
+	    $fh->{base} = [$self->_retrieve_base($path, $pool)];
 	}
 	# get new
 	$fh->{new} = [tmpfile('new-')];
-	return [SVN::TxDelta::apply ($base, $fh->{new}[FH], undef, undef, $pool)];
+	return [SVN::TxDelta::apply ($fh->{base}[FH], $fh->{new}[FH], undef, undef, $pool)];
     }
     $self->{notify}->node_status ($path, 'U')
 	unless $self->{notify}->node_status ($path);
 
     $self->ensure_open ($path);
-    return $self->{storage}->apply_textdelta ($self->{storage_baton}{$path},
-					      $checksum, $pool);
+    my $handle = $self->{storage}->apply_textdelta ($self->{storage_baton}{$path},
+						    $checksum, $pool);
+    if ($self->{storage_has_unwritable} && !$handle) {
+	delete $self->{notify}{status}{$path};
+	$self->{notify}->flush ($path);
+    }
+    return $handle;
 }
 
 sub _merge_text_change {
@@ -371,6 +390,40 @@ sub _merge_text_change {
     return ($conflict, $mfh);
 }
 
+sub _overwrite_local_file {
+    my ($self, $fh, $path, $nfh, $pool) = @_;
+    my $handle = $self->{storage}->
+	apply_textdelta ($self->{storage_baton}{$path}, $fh->{local}[CHECKSUM],
+			 $pool);
+
+    if ($handle && $#{$handle} >= 0) {
+	if ($self->{send_fulltext}) {
+	    SVN::TxDelta::send_stream ($nfh, @$handle, $pool);
+	}
+	else {
+	    seek $fh->{local}[FH], 0, 0;
+	    my $txstream = SVN::TxDelta::new($fh->{local}[FH], $nfh, $pool);
+	    SVN::TxDelta::send_txstream ($txstream, @$handle, $pool);
+	}
+	return 1;
+    }
+
+    if ($self->{storage_has_unwritable}) {
+	delete $self->{notify}{status}{$path};
+	$self->{notify}->flush ($path);
+	return 0;
+    }
+    return 1;
+}
+
+sub _merge_file_unchanged {
+    my ($self, $path, $checksum, $pool) = @_;
+    ++$self->{changes};
+    $self->{notify}->node_status ($path, 'g');
+    $self->ensure_close ($path, $checksum, $pool);
+    return;
+}
+
 sub close_file {
     my ($self, $path, $checksum, $pool) = @_;
     return unless $path;
@@ -381,15 +434,18 @@ sub close_file {
     no warnings 'uninitialized';
     # let close_directory reports about its children
     if ($info->{fh}{new}) {
-	$self->prepare_fh ($fh);
 
-	if ($checksum eq $fh->{local}[CHECKSUM] ||
-	    File::Compare::compare ($fh->{new}[FILENAME], $fh->{local}[FILENAME]) == 0) {
-	    ++$self->{changes};
-	    $self->{notify}->node_status ($path, 'g');
-	    $self->ensure_close ($path, $checksum, $pool);
-	    return;
-	}
+	$self->_merge_file_unchanged ($path, $checksum, $pool), return
+	    if $checksum eq $fh->{local}[CHECKSUM];
+
+	my $eol = $self->{cb_localprop}->($path, 'svn:eol-style', $pool);
+	my $eol_layer = SVK::XD::get_eol_layer({'svn:eol-style' => $eol}, '>');
+	$eol_layer = '' if $eol_layer eq ':raw';
+	$self->prepare_fh ($fh, $eol_layer);
+	# XXX: There used be a case that this explicit comparison is
+	# needed, but i'm not sure anymore.
+	$self->_merge_file_unchanged ($path, $checksum, $pool), return
+	    if File::Compare::compare ($fh->{new}[FILENAME], $fh->{local}[FILENAME]) == 0;
 
 	$self->ensure_open ($path);
         if ($info->{addmerge}) {
@@ -398,29 +454,17 @@ sub close_file {
         }
 	my ($conflict, $mfh) = $self->_merge_text_change ($fh, $path, $pool);
 	$self->{notify}->node_status ($path, $conflict ? 'C' : 'G');
+
+	$eol_layer = SVK::XD::get_eol_layer({'svn:eol-style' => $eol}, '<');
+	binmode $mfh, $eol_layer or die $! if $eol_layer;
+
 	$iod = IO::Digest->new ($mfh, 'MD5');
 
-	my $handle = $self->{storage}->
-	    apply_textdelta ($self->{storage_baton}{$path}, $fh->{local}[CHECKSUM],
-			     $pool);
-
-	if ($handle && $#{$handle} >= 0) {
-	    if ($self->{send_fulltext}) {
-		SVN::TxDelta::send_stream ($mfh, @$handle, $pool)
-			if $handle && $#{$handle} >= 0;
-	    }
-	    else {
-		seek $fh->{local}[FH], 0, 0;
-		my $txstream = SVN::TxDelta::new
-		    ($fh->{local}[FH], $mfh, $pool);
-		SVN::TxDelta::send_txstream ($txstream, @$handle, $pool);
-	    }
+	if ($self->_overwrite_local_file ($fh, $path, $mfh, $pool)) {
+	    undef $fh->{base}[FILENAME] if $info->{addmerge};
+	    $self->node_conflict ($path) if $conflict;
 	}
-
-	undef $fh->{base}[FILENAME] if $info->{addmerge};
 	$self->cleanup_fh ($fh);
-
-	$self->node_conflict ($path) if $conflict;
     }
     elsif ($info->{fpool} && !$self->{notify}->node_status ($path)) {
 	# open but prop edit only, load local checksum
@@ -450,10 +494,15 @@ sub add_directory {
 	$self->{notify}->node_status ($path, 'G');
     }
     else {
-	$self->{added}{$path} = 1;
-	$self->{storage_baton}{$path} =
+	my $baton =
 	    $self->{storage}->add_directory ($path, $self->{storage_baton}{$pdir},
 					     @arg);
+	unless (defined $baton) {
+	    $self->{notify}->flush ($path);
+	    return undef;
+	}
+	$self->{storage_baton}{$path} = $baton;
+	$self->{added}{$path} = 1;
 	$self->{notify}->node_status ($path, 'A');
 	$self->{notify}->flush ($path, 1);
     }
@@ -494,15 +543,51 @@ sub close_directory {
 	unless $path eq '';
 }
 
+sub _merge_file_delete {
+    my ($self, $path, $rpath, $pdir, $pool) = @_;
+    return undef unless $self->{cb_localmod}->(
+		$path,
+		$self->{base_root}->file_md5_checksum ($rpath, $pool),
+		$pool);
+    return {} unless $self->{resolve};
+
+    my $fh = $self->{info}{$path}->{fh} || {};
+    $fh->{base} ||= [$self->_retrieve_base($path, $pool)];
+    $fh->{new} = [tmpfile('new-')];
+    $fh->{local} = [tmpfile('local-')];
+    my ($tmp) = $self->{cb_localmod}->($path, '', $pool);
+    slurp_fh ( $tmp->[FH], $fh->{local}[FH]);
+    seek $fh->{local}[FH], 0, 0;
+    $fh->{local}[CHECKSUM] = $tmp->[CHECKSUM];
+
+    my ($conflict, $mfh) = $self->_merge_text_change( $fh, $path, $pool);
+    if( $conflict ) {
+	$self->clean_up($fh);
+	return {};
+    } elsif( !(stat($mfh))[7] ) {
+	#delete file if merged size is 0
+	$self->clean_up($fh);
+	return undef;
+    }
+    seek $mfh, 0, 0;
+    my $iod = IO::Digest->new ($mfh, 'MD5');
+
+    $self->{info}{$path}{open} = [$pdir, -1];
+    $self->{info}{$path}{fpool} = $pool;
+    $self->ensure_open ($path);
+    $self->_overwrite_local_file ($fh, $path, $mfh, $pool);
+    ++$self->{changes};
+    $self->ensure_close ($path, $iod->hexdigest, $pool);
+
+    return 1;
+}
 # returns undef for deleting this, a hash for partial delete.
 # returns 1 for merged delete
 # Note that empty hash means don't delete.
 sub _check_delete_conflict {
     my ($self, $path, $rpath, $kind, $pdir, $pool) = @_;
-    return $self->{cb_localmod}->
-	($path, $self->{base_root}->file_md5_checksum ($rpath, $pool), $pool)
-	    ? {} : undef
-		if $kind == $SVN::Node::file;
+
+    return $self->_merge_file_delete($path, $rpath, $pdir, $pool) if $kind == $SVN::Node::file;
 
     my $dirmodified = $self->{cb_dirdelta}->($path, $self->{base_root}, $rpath);
     my $entries = $self->{base_root}->dir_entries ($rpath);
@@ -583,11 +668,12 @@ sub delete_entry {
     my $torm = $self->_check_delete_conflict ($path, $rpath,
 					      $self->{base_root}->check_path ($rpath), $pdir, @arg);
 
-    if ($torm) {
+    if (ref($torm)) {
 	$self->node_conflict ($path);
 	$self->_partial_delete ($torm, $path, $self->{storage_baton}{$pdir}, @arg);
-    }
-    else {
+    } elsif( $torm && $torm == 1) {
+	$self->{notify}->node_status ($path, 'G');
+    } else {
 	$self->{storage}->delete_entry ($path, $self->{cb_rev}->($path),
 					$self->{storage_baton}{$pdir}, @arg);
 	$self->{notify}->node_status ($path, 'D');
@@ -648,8 +734,9 @@ sub _merge_prop_change {
     $prop->{new} = $_[1];
     {
 	local $@;
-	$prop->{base} = eval { $self->{base_root}->node_prop ($rpath, $_[0]) };
-	$prop->{local} = $self->{cb_exist}->($path) ? $self->{cb_localprop}->($path, $_[0]) : undef;
+	$prop->{base} = eval { $self->{base_root}->node_prop ($rpath, $_[0], $pool) };
+	$prop->{local} = $self->{cb_exist}->($path)
+	    ? $self->{cb_localprop}->($path, $_[0], $pool) : undef;
     }
     # XXX: only known props should be auto-merged with default resolver
     $pool = pop @_ if ref ($_[-1]) =~ m/^(?:SVN::Pool|_p_apr_pool_t)$/;
